#include <stdio.h>
#include <stdlib.h>

#define JOBQ     5              /* Number of jobs in the hold queue */
#define QUANTUM 10              /* time quantum per process */

/* Process Control Block */
struct Job {
        int     pid;
        int     t;
        int     cpucycle;
        char    run[256];
        int     active;
};

int
main(int argc, char **argv)
{
        /*
         * Allocate contiguous block of memory for job queue that contains
         * pointers to each PCB, and initialise a pointer to the queue.
         */
        struct Job      ***p;
        struct Job       **queue;
        int                ct;
        int                ready;
        int                time, tq;
        
        queue = calloc(JOBQ, sizeof(struct Job *));
        p = &queue;
        ct = 0;
        ready = 1;
        time = 0;
        tq = 0;
        
        /* Allocate memory for each PCB and initialise each block to zero */
        for (int i = 0; i < JOBQ; ++i) {
                queue[i] = calloc(1, sizeof(struct Job));
        }

        /* Load data into each PCB */
        queue[0]->pid = 0;
        queue[0]->t = 0;
        queue[0]->cpucycle = 26;

        queue[1]->pid = 1;
        queue[1]->t = 0;
        queue[1]->cpucycle = 30;
        
        queue[2]->pid = 2;
        queue[2]->t = 0;
        queue[2]->cpucycle = 89;
        
        queue[3]->pid = 3;
        queue[3]->t = 0;
        queue[3]->cpucycle = 16;
        
        queue[4]->pid = 4;
        queue[4]->t = 0;
        queue[4]->cpucycle = 40;
        
        /*
         * Iterate over queue, if job has arrived and has cpu cycles: set
         * active bit; while time quantum hasn't elapsed and active job has
         * cpu cycles remaining, increment its run counter and all inactive
         * jobs' wait counters; each cpu cycle, increment job run counter, the
         * time quantum, and time; and decrement active job's cpu cycles. Once
         * time quantum has elapsed, reset quantum, and unset job active bit.
         * Continue till queue is empty of unfinished jobs. When finished,
         * display to screen progression of jobs throughout queue.
         */
        do {
                for (int i = 0; i < JOBQ; ++i) {
                        if ((*p)[i]->t <= time && (*p)[i]->cpucycle > 0) {
                                (*p)[i]->active = 1;
                                while (tq < QUANTUM && (*p)[i]->cpucycle > 0) {
                                        (*p)[i]->run[ct] = '#';
                                        for (int x = 0; x < JOBQ; ++x)
                                                if (!(*p)[x]->active)
                                                        (*p)[x]->run[ct] = '-';
                                        ++ct;
                                        ++time;
                                        ++tq;
                                        (*p)[i]->cpucycle--;
                                        
                                }
                                
                                tq = 0;
                                (*p)[i]->active = 0;
                                ready = 0;
                        }
                        for (int i = 0; i < JOBQ; ++i)
                                if ((*p)[i]->cpucycle > 0)
                                        ready = 1;
                }
        } while (ready == 1);

        for (int i = 0; i < JOBQ; ++i)
                printf("%s\n", (*p)[i]->run);
}